import sir.helpers.api
from flask import g


def get_dates(request):
    db = getattr(g, 'db')
    dates = [d.strftime('%Y-%m-%dT%H:%M:%S') for d in db.get_dates()]
    return sir.helpers.api.build_api_response(result=dates, error=False)


def get_flows(request):
    db = getattr(g, 'db')
    start_time = request.args.get('start_time')
    end_time = request.args.get('end_time')
    flows = db.get_flows(start_time, end_time)
    return sir.helpers.api.build_api_response(flows, error=False, start_time=start_time, end_time=end_time)


def get_bgp_prefixes(request):
    fs = getattr(g, 'fs')
    date = request.args.get('date')
    bgp_prefixes = fs.get_bgp_prefixes(date)
    return sir.helpers.api.build_api_response(bgp_prefixes, error=False, date=date)

def get_raw_bgp(request):
    fs = getattr(g, 'fs')
    date = request.args.get('date')
    bgp_prefixes = fs.get_raw_bgp(date)
    return sir.helpers.api.build_api_response(bgp_prefixes, error=False, date=date)



from flask import render_template
from flask import g


def _init_context_dates(db, request):
    context = dict()
    dates = db.get_dates()

    starting_time = min(len(dates), 25)

    context['avail_start_time'] = dates[0].strftime('%Y-%m-%dT%H:%M')
    context['avail_end_time'] = dates[-1].strftime('%Y-%m-%dT%H:%M')
    context['start_time'] = request.form.get('start_time', dates[-starting_time].strftime('%Y-%m-%dT%H:%M'))
    context['end_time'] = request.form.get('end_time', context['avail_end_time'])
    return context


def start_page(request):
    return render_template('analytics/start_page.html')


def offloaded_traffic(request):
    db = getattr(g, 'db', None)
    context = _init_context_dates(db, request)

    context['num_prefixes'] = int(request.form.get('num_prefixes', 1000))

    if request.method == 'GET':
        context['total_bytes'] = 0
        context['offloaded_bytes'] = 0
        context['percentage'] = 0.0
    elif request.method == 'POST':
        context['total_bytes'] = db.get_total_traffic(context['start_time'], context['end_time'])
        context['offloaded_bytes'
               ] = db.offloaded_bytes(context['num_prefixes'], context['start_time'], context['end_time'])
        context['percentage'] = float(context['offloaded_bytes']) * 100.0 / float(context['total_bytes'])

    return render_template('analytics/offloaded_traffic.html', **context)


def aggregate(request, field):
    db = getattr(g, 'db', None)
    context = _init_context_dates(db, request)

    context['flow_aggr'] = list()
    context['time_series'] = dict()
    context['time_series_times'] = list()

    if field == 'as':
        aggregate_method = db.aggregate_per_as
        timeseries_method = db.timeseries_per_as
        context['title'] = 'ASN\'s'
    elif field == 'prefix':
        aggregate_method = db.aggregate_per_prefix
        timeseries_method = db.timeseries_per_prefix
        context['title'] = 'Prefixes'

    if request.method == 'POST':
        context['time_series_times'] = db.get_dates_in_range(context['start_time'], context['end_time'])

        context['flow_aggr'] = aggregate_method(context['start_time'], context['end_time'])
        time_series = dict()
        for a in context['flow_aggr'][0:10]:
            time_series[a['key']] = timeseries_method(context['start_time'], context['end_time'], a['key'])

        context['time_series'] = time_series

    return render_template('analytics/analytics_aggregate.html', **context)


def simulate(request):
    db = getattr(g, 'db', None)
    context = _init_context_dates(db, request)

    context['num_prefixes'] = int(request.form.get('num_prefixes', 1000))
    context['time_series'] = dict()
    context['time_series_times'] = list()

    if request.method == 'POST':
        context['time_series_times'] = db.get_dates_in_range(context['start_time'], context['end_time'])

        time_series = dict()
        time_series['total_bytes'] = list()
        time_series['offloaded_bytes'] = list()

        for time_serie in context['time_series_times']:
            time_series['total_bytes'].append(db.get_total_traffic(time_serie, time_serie))
            time_series['offloaded_bytes'].append(db.offloaded_bytes(context['num_prefixes'], time_serie, time_serie))

        context['time_series'] = time_series

    return render_template('analytics/simulate.html', **context)


def find_prefix(request):
    fs = getattr(g, 'fs', None)
    context = dict()
    context['available_dates'] = [d.strftime('%Y-%m-%dT%H:%M:01') for d in fs.get_available_dates()]

    context['query_name'] = 'Prefix'
    if request.method == 'GET':
        context['query'] = ''
        context['prefixes'] = dict()
        context['date'] = context['available_dates'][-1]
    elif request.method == 'POST':
        context['date'] = request.form.get('date')
        context['query'] = request.form.get('query')
        context['prefixes'] = fs.find_prefix(context['query'], context['date'])
    return render_template('analytics/find_prefix.html', **context)


def find_prefix_asn(request):
    fs = getattr(g, 'fs', None)
    context = dict()
    context['available_dates'] = [d.strftime('%Y-%m-%dT%H:%M:01') for d in fs.get_available_dates()]

    context['query_name'] = 'ASN'
    if request.method == 'GET':
        context['query'] = ''
        context['prefixes'] = dict()
        context['origin_only'] = True
        context['date'] = context['available_dates'][-1]
    elif request.method == 'POST':
        context['date'] = request.form.get('date')        
        context['query'] = request.form.get('query')
        context['origin_only'] = eval(request.form.get('origin_only', True))
        context['prefixes'] = fs.find_prefixes_asn(context['query'], context['date'], context['origin_only'])
    return render_template('analytics/find_prefix.html', **context)



import sqlite3 as lite
import os
from datetime import datetime


def dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d


class SQLite3Helper:

    def __init__(self, database):
        if not os.path.isfile(database):
            raise Exception("Database file doesn't exist: %s" % database)

        self.database = database
        self.conn = None

    def connect(self):
        self.conn = lite.connect(database=self.database)
        self.conn.row_factory = dict_factory

    def close(self):
        if self.conn is not None:
            self.conn.close()

    def _execute_query(self, query, args=(), commit=False):
        try:
            cur = self.conn.cursor()
            cur.execute(query, args)

            if commit:
                result = self.conn.commit()
            else:
                result = cur.fetchall()
        except lite.OperationalError:
            raise Exception('The following query failed:\n%s' % query)
        return result

    def aggregate_per_prefix(self, start_time, end_time, limit=0, net_masks='', exclude_net_masks=False):
        """ Given a time range aggregates bytes per prefix.

            Args:
                start_time: A string representing the starting time of the time range
                end_time: A string representing the ending time of the time range
                limit: An optional integer. If it's >0 it will limit the amount of prefixes returned.

            Returns:
                A list of prefixes sorted by sum_bytes. For example:

                [
                        {'key': '192.168.1.0/25', 'sum_bytes': 3000, 'as_dst': 345},
                        {'key': '192.213.1.0/25', 'sum_bytes': 2000, 'as_dst': 123},
                        {'key': '231.168.1.0/25', 'sum_bytes': 1000, 'as_dst': 321},
                ]
        """
        if net_masks == '':
            net_mask_filter = ''
        elif not exclude_net_masks:
            net_mask_filter = 'AND mask_dst IN ({})'.format(net_masks)
        elif exclude_net_masks:
            net_mask_filter = 'AND mask_dst NOT IN ({})'.format(net_masks)

        query = ''' SELECT ip_dst||'/'||mask_dst as key, SUM(bytes) as sum_bytes, as_dst
                    from acct
                    WHERE
                    datetime(stamp_updated) BETWEEN datetime(?) AND datetime(?, "+1 second")
                    {}
                    GROUP by ip_dst,mask_dst
                    ORDER BY SUM(bytes) DESC
                '''.format(net_mask_filter)

        if limit > 0:
            query += 'LIMIT %d' % limit

        return self._execute_query(query, [start_time, end_time])

    def aggregate_per_as(self, start_time, end_time):
        """ Given a time range aggregates bytes per ASNs.

            Args:
                start_time: A string representing the starting time of the time range
                end_time: A string representing the ending time of the time range

            Returns:
                A list of prefixes sorted by sum_bytes. For example:

                [
                        {'key': '6500', 'sum_bytes': 3000},
                        {'key': '2310', 'sum_bytes': 2000},
                        {'key': '8182', 'sum_bytes': 1000},
                ]
        """

        query = ''' SELECT as_dst as key, SUM(bytes) as sum_bytes
                    from acct
                    WHERE
                    datetime(stamp_updated) BETWEEN datetime(?) AND datetime(?, "+1 second")
                    GROUP by as_dst
                    ORDER BY SUM(bytes) DESC;
                '''

        return self._execute_query(query, [start_time, end_time])

    def get_dates(self):
        query = '''SELECT DISTINCT stamp_updated from acct ORDER BY stamp_updated ASC;'''
        return [datetime.strptime(d['stamp_updated'], '%Y-%m-%d %H:%M:%S') for d in self._execute_query(query)]

    def get_dates_in_range(self, start_time, end_time):
        query = ''' SELECT DISTINCT stamp_updated
                    from acct
                    WHERE
                    datetime(stamp_updated) BETWEEN datetime(?) AND datetime(?, "+1 second")
                    ;
                '''

        return [datetime.strptime(d['stamp_updated'], '%Y-%m-%d %H:%M:%S')
                for d in self._execute_query(query, [start_time, end_time])]

    def get_total_traffic(self, start_time, end_time):
        query = ''' SELECT SUM(bytes) as sum_bytes
                    FROM acct
                    WHERE
                    datetime(stamp_updated) BETWEEN datetime(?) AND datetime(?, "+1 second")
                    ;
                '''

        return self._execute_query(query, [start_time, end_time])[0]['sum_bytes']

    def offloaded_bytes(self, num_prefixes, start_time, end_time):
        query = ''' SELECT SUM(bytes) as sum_bytes FROM (
                        SELECT ip_dst, mask_dst, SUM(bytes) AS bytes
                        from acct
                        WHERE
                        datetime(stamp_updated) BETWEEN datetime(:start_time) AND datetime(:end_time, "+1 second")
                        GROUP BY ip_dst, mask_dst
                        ORDER BY SUM(bytes) DESC
                        LIMIT %d
                    );
                ''' % num_prefixes
        args = {'start_time': start_time, 'end_time': end_time, 'num_prefixes': num_prefixes,}
        return self._execute_query(query, args)[0]['sum_bytes']

    def timeseries_per_as(self, start_time, end_time, asn):
        query = ''' SELECT SUM(bytes) as sum_bytes
                    from acct
                    WHERE
                    datetime(stamp_updated) BETWEEN datetime(?) AND datetime(?, "+1 second")
                    AND
                    as_dst = ?
                    GROUP by as_dst, stamp_updated
                    ORDER BY stamp_updated ASC;
                '''

        return [r['sum_bytes'] for r in self._execute_query(query, [start_time, end_time, asn])]

    def timeseries_per_prefix(self, start_time, end_time, prefix):
        ip_dst, mask_dst = prefix.split('/')
        query = ''' SELECT SUM(bytes) as sum_bytes
                    from acct
                    WHERE
                    datetime(stamp_updated) BETWEEN datetime(?) AND datetime(?, "+1 second")
                    AND
                    ip_dst = ? AND mask_dst = ?
                    GROUP by ip_dst, mask_dst, stamp_updated
                    ORDER BY stamp_updated ASC;
                '''

        return [r['sum_bytes'] for r in self._execute_query(query, [start_time, end_time, ip_dst, mask_dst])]

    def put_variables(self, name, content, category):
        query = ''' INSERT INTO variables VALUES (?, ?, ?); '''
        self._execute_query(query, [name, content, category], commit=True)

    def get_variables(self):
        query = ''' SELECT * FROM variables '''
        return self._execute_query(query)

    def get_variable(self, category, name):
        query = ''' SELECT * FROM variables WHERE category = ? AND name = ?; '''
        return self._execute_query(query, [category, name])

    def update_variable(self, old_name, old_category, name, content, category):
        query = ''' UPDATE variables
                    SET name = ?, content = ?, category = ?
                    WHERE name = ? AND category = ?;
                '''
        self._execute_query(query, [name, content, category, old_name, old_category], commit=True)

    def delete_variable(self, category, name):
        query = ''' DELETE FROM variables WHERE name = ? AND category = ?;
                '''
        self._execute_query(query, [name, category], commit=True)

    def get_categories(self):
        query = ''' SELECT DISTINCT category FROM variables'''
        return [c['category'] for c in self._execute_query(query, [])]

    def filter_variables_category(self, category):
        query = ''' SELECT * FROM variables WHERE category = ?; '''
        return self._execute_query(query, [category])

    def get_flows(self, start_time, end_time):
        query = ''' SELECT ip_dst, mask_dst, bytes, packets, as_dst, stamp_updated
                            FROM acct
                            WHERE
                            datetime(stamp_updated) BETWEEN datetime(?) AND datetime(?, "+1 second")
                            ;
                '''
        return self._execute_query(query, [start_time, end_time])



from flask import render_template
from flask import g

import yaml


def start_page(request):
    context = dict()
    with open('api/api_documentation.yaml', 'r') as stream:
        context['documentation'] = yaml.load(stream)

    return render_template('api/start_page.html', **context)



# -*- coding: utf-8 -*-

from sir.helpers.SQLite3Helper import SQLite3Helper
from sir.helpers.FSHelper import FSHelper

import sir.variables.api
import sir.variables.views

import sir.analytics.api
import sir.analytics.views

import sir.api.views

import sir.pmacct_data.api

from flask import Flask, request, g, jsonify, render_template

import time

app = Flask(__name__)
app.config.from_envvar('SIR_SETTINGS', silent=True)

###################
###################
#  BASIC  #########
###################
###################


@app.before_request
def before_request():
    g.db = SQLite3Helper(app.config['DATABASE'])
    g.db.connect()
    g.request_start_time = time.time()
    g.request_time = lambda: float("%.5f" %
                                   (time.time() - g.request_start_time))
    g.fs = FSHelper(app.config['BGP_FOLDER'])


@app.teardown_request
def teardown_request(exception):
    db = getattr(g, 'db', None)
    if db is not None:
        db.close()


@app.route('/', strict_slashes=False)
def start_page():
    return render_template('basic/start_page.html')

###################
###################
#  ANALYTICS  #####
###################
###################


@app.route('/analytics', strict_slashes=False)
def analytics_view_help():
    return sir.analytics.views.start_page(request)


@app.route('/analytics/offloaded_traffic', methods=['GET', 'POST'])
def analytics_view_offloaded_traffic():
    return sir.analytics.views.offloaded_traffic(request)


@app.route('/analytics/aggregate_per_as', methods=['GET', 'POST'])
def analytics_view_aggregate_per_as():
    return sir.analytics.views.aggregate(request, 'as')


@app.route('/analytics/aggregate_per_prefix', methods=['GET', 'POST'])
def analytics_view_aggregate_per_prefix():
    return sir.analytics.views.aggregate(request, 'prefix')


@app.route('/analytics/simulate', methods=['GET', 'POST'])
def analytics_view_simulate():
    return sir.analytics.views.simulate(request)


@app.route('/api/v1.0/analytics/top_prefixes', methods=['GET'])
def analytics_api_top_prefixes():
    return jsonify(sir.analytics.api.top_prefixes(request))


@app.route('/api/v1.0/analytics/top_asns', methods=['GET'])
def analytics_api_top_asns():
    return jsonify(sir.analytics.api.top_asns(request))


@app.route('/api/v1.0/analytics/find_prefix/<prefix>/<pl>', methods=['GET'])
def analytics_api_find_prefix(prefix, pl):
    return jsonify(sir.analytics.api.find_prefix(request, u'{}/{}'.format(prefix, pl)))


@app.route('/analytics/find_prefix', methods=['GET', 'POST'])
def analytics_view_find_prefix():
    return sir.analytics.views.find_prefix(request)


@app.route('/api/v1.0/analytics/find_prefixes_asn/<asn>', methods=['GET'])
def analytics_api_find_prefixes_asn(asn):
    return jsonify(sir.analytics.api.find_prefixes_asn(request, asn))


@app.route('/analytics/find_prefixes_asn', methods=['GET', 'POST'])
def analytics_view_find_prefix_asn():
    return sir.analytics.views.find_prefix_asn(request)


###################
###################
#  API  ###########
###################
###################


@app.route('/api/documentation', strict_slashes=False)
def api_help():
    return sir.api.views.start_page(request)


###################
###################
#  VARIABLES  #####
###################
###################


@app.route('/variables/browse', methods=['GET'])
def browse_view_variables():
    return sir.variables.views.browse_variables(request)

'''
@app.route('/variables/edit/<category>/<name>', methods=['GET', 'POST', 'DELETE'])
def edit_variable(category, name):
    return variables.views.edit_variable(request, category, name)
'''


@app.route('/api/v1.0/variables', methods=['GET', 'POST'])
def variables_api_variables():
    return jsonify(sir.variables.api.variables(request))


@app.route('/api/v1.0/variables/categories', methods=['GET'])
def variables_api_category():
    return jsonify(sir.variables.api.variables_category(request))


@app.route('/api/v1.0/variables/categories/<category>', methods=['GET'])
def variables_api_filter_by_category(category):
    return jsonify(sir.variables.api.variables_filter_by_category(request, category))


@app.route('/api/v1.0/variables/categories/<category>/<name>', methods=['GET', 'PUT', 'DELETE'])
def variables_api_name(category, name):
    return jsonify(sir.variables.api.api_variables_name(request, category, name))

###################
###################
#  PMACCT_DATA  ###
###################
###################


@app.route('/api/v1.0/pmacct/dates', methods=['GET'])
def pmacct_data_api_get_dates():
    return jsonify(sir.pmacct_data.api.get_dates(request))


@app.route('/api/v1.0/pmacct/flows', methods=['GET'])
def pmacct_data_api_get_flows():
    return jsonify(sir.pmacct_data.api.get_flows(request))


@app.route('/api/v1.0/pmacct/raw_bgp', methods=['GET'])
def pmacct_data_api_get_raw_bgp():
    return jsonify(sir.pmacct_data.api.get_raw_bgp(request))


@app.route('/api/v1.0/pmacct/bgp_prefixes', methods=['GET'])
def pmacct_data_api_get_bgp_prefixes():
    return jsonify(sir.pmacct_data.api.get_bgp_prefixes(request))

if __name__ == '__main__':
    app.run(app.config['BIND_IP'], app.config['PORT'])



import sir.helpers.api
from flask import g


def top_prefixes(request):
    # curl http://127.0.0.1:5000/api/v1.0/top_prefixes\?limit_prefixes=10\&start_time\=2015-07-13T14:00\&end_time\=2015-07-14T14:00
    db = getattr(g, 'db')
    start_time = request.args.get('start_time')
    end_time = request.args.get('end_time')
    limit_prefixes = int(request.args.get('limit_prefixes', 0))
    net_masks = request.args.get('net_masks', '')
    exclude_net_masks = request.args.get('exclude_net_masks', False)

    result = db.aggregate_per_prefix(
        start_time, end_time,
        limit=limit_prefixes,
        net_masks=net_masks,
        exclude_net_masks=exclude_net_masks)

    parameters = {
        'limit_prefixes': limit_prefixes,
        'start_time': start_time,
        'end_time': end_time,
        'net_masks': net_masks,
        'exclude_net_masks': exclude_net_masks,
    }
    return sir.helpers.api.build_api_response(result, error=False, **parameters)


def top_asns(request):
    # curl http://127.0.0.1:5000/api/v1.0/top_asns\?start_time=2015-07-13T14:00\&end_time=2015-07-14T14:00
    db = getattr(g, 'db')
    start_time = request.args.get('start_time')
    end_time = request.args.get('end_time')

    result = db.aggregate_per_as(start_time, end_time)
    parameters = {
        'start_time': start_time,
        'end_time': end_time,
    }
    return sir.helpers.api.build_api_response(result, error=False, **parameters)


def find_prefix(request, prefix):
    # curl http://127.0.0.1:5000/api/v1.0/top_asns\?start_time=2015-07-13T14:00\&end_time=2015-07-14T14:00
    fs = getattr(g, 'fs')
    date = request.args.get('date')
    print date
    result = fs.find_prefix(prefix, date)
    parameters = {
        'prefix': prefix,
        'date': date,
    }
    return sir.helpers.api.build_api_response(result, error=False, **parameters)


def find_prefixes_asn(request, asn):
    # curl http://127.0.0.1:5000/api/v1.0/top_asns\?start_time=2015-07-13T14:00\&end_time=2015-07-14T14:00
    fs = getattr(g, 'fs')
    date = request.args.get('date')
    origin_only = request.args.get('origin_only', False)

    result = fs.find_prefixes_asn(asn, date, origin_only)
    parameters = {
        'asn': asn,
        'date': date,
        'origin_only': origin_only,
    }
    return sir.helpers.api.build_api_response(result, error=False, **parameters)



import glob
from datetime import datetime
import json
import ipaddress


class FSHelper:
    def __init__(self, base_path='./data'):
        self.base_path = base_path
        files = [f.split('/')[-1] for f in glob.glob('{}/bgp-*'.format(self.base_path))]
        self.neighbors = set([n.split('-')[1].replace('_', '.') for n in files])
        dates = sorted(set([n.split('-')[2].replace('.txt', '') for n in files]))
        self.dates = [datetime.strptime(d, '%Y_%m_%dT%H_%M_%S') for d in dates]

    def get_available_dates(self):
        return self.dates

    def get_bgp_prefixes(self, date):
        date = date.replace('-', '_').replace(':', '_')

        prefixes = dict()

        for n in self.neighbors:
            prefixes[n] = list()
            f = 'bgp-{}-{}'.format(n, date).replace('.', '_')

            with open('{}/{}.txt'.format(self.base_path, f)) as data_file:
                for line in data_file.readlines():
                    json_data = json.loads(line)
                    if json_data['event_type'] == 'dump':
                        prefixes[n].append(json_data['ip_prefix'])
                data_file.close()

        return prefixes

    def get_raw_bgp(self, date):
        date = date.replace('-', '_').replace(':', '_')

        prefixes = dict()

        for n in self.neighbors:
            prefixes[n] = dict()
            f = 'bgp-{}-{}'.format(n, date).replace('.', '_')

            with open('{}/{}.txt'.format(self.base_path, f)) as data_file:
                prefixes[n] = data_file.read()
                data_file.close()

        return prefixes

    def find_prefix(self, prefix, date):
        ip_prefix_str = unicode(prefix)
        search_string = '"ip_prefix": "{}.'.format(ip_prefix_str.split('.')[0])
        ipp = ipaddress.ip_network(ip_prefix_str)

        date = date.replace('-', '_').replace(':', '_')

        prefixes = dict()

        for n in self.neighbors:
            prefixes[n] = list()
            f = 'bgp-{}-{}'.format(n, date).replace('.', '_')

            with open('{}/{}.txt'.format(self.base_path, f)) as f:
                for line in f:
                    if search_string in line:
                        data = json.loads(line)
                        other_ipp = ipaddress.ip_network(unicode(data['ip_prefix']))
                        if ipp.overlaps(other_ipp):
                            if ipp.prefixlen >= other_ipp.prefixlen:
                                prefixes[n].append(data)
        return prefixes

    def find_prefixes_asn(self, asn, date, originate_only=False):
        date = date.replace('-', '_').replace(':', '_')

        prefixes = dict()

        for n in self.neighbors:
            prefixes[n] = list()
            f = 'bgp-{}-{}'.format(n, date).replace('.', '_')

            with open('{}/{}.txt'.format(self.base_path, f)) as f:
                for line in f:
                    if asn in line:
                        data = json.loads(line)
                        as_path = data['as_path'].split(' ')
                        if originate_only:
                            if asn == as_path[-1]:
                                prefixes[n].append(data)
                        else:
                            if asn in as_path:
                                prefixes[n].append(data)
        return prefixes



from flask import render_template
from flask import g

def browse_variables(request):
    db = getattr(g, 'db', None)
    context = dict()

    context['categories'] = db.get_categories()

    context['filter_category'] = request.args.get('category', None)

    if context['filter_category'] is None:
        context['variables'] = db.get_variables()
    else:
        context['variables'] = db.filter_variables_category(context['filter_category'])
    return render_template('variables/browse.html', **context)

'''
def edit_variable(request, orig_category, orig_name):
    db = getattr(g, 'db', None)
    context = dict()
    context['saved'] = False
    context['deleted'] = False
    context['orig_category'] = orig_category
    context['orig_name'] = orig_name

    if request.method == 'GET':
        context['variable'] = db.get_variable(orig_category, orig_name)
    elif request.method == 'POST':
        category = request.form.get('category')
        name = request.form.get('name')
        content = request.form.get('content')
        extra_vars = request.form.get('extra_vars')
        db.update_variable(orig_name, orig_category, name, content, category, extra_vars)
        context['saved'] = True
        context['variable'] = db.get_variable(orig_category, orig_name)

    return render_template('variables/edit.html', **context)
'''



from flask import g
import sir.helpers.api


def _variables_post(request):
    # curl -i -H "Content-Type: application/json" -X POST -d '{"name": "test", "category": "development", "content": {"ads": "qwe", "asd": "zxc"}}' http://127.0.0.1:5000/api/v1.0/variables
    db = getattr(g, 'db')

    name = request.json.get('name')
    content = request.json.get('content')
    category = request.json.get('category')

    db.put_variables(name, content, category)

    result = db.get_variable(category, name)
    parameters = {
        'name': name,
        'content': content,
        'category': category,
    }
    return sir.helpers.api.build_api_response(result, error=False, **parameters)


def _variables_get(request):
    db = getattr(g, 'db')
    result = db.get_variables()
    parameters = {}
    return sir.helpers.api.build_api_response(result, error=False, **parameters)


def variables(request):
    if request.method == 'GET':
        return _variables_get(request)
    elif request.method == 'POST':
        return _variables_post(request)


def _api_variables_var_id_get(request, name):
    db = getattr(g, 'db')
    result = db.get_variable(name)
    parameters = {
        'name': name,
    }
    return sir.helpers.api.build_api_response(result, error=False, **parameters)


def api_variables_name(request, category, name):
    db = getattr(g, 'db')

    if request.method == 'GET':
        result = db.get_variable(category, name)
    elif request.method == 'PUT':
        # curl -i -H "Content-Type: application/json" -X PUT -d '{"name": "test", "category": "development", "content": {"ads": "qwe", "asd": "zxc"}}' http://127.0.0.1:5000/api/v1.0/variables/test
        variable = db.get_variable(category, name)[0]
        new_name = request.json.get('name', variable['name'])
        new_content = request.json.get('content', variable['content'])
        new_category = request.json.get('category', variable['category'])
        db.update_variable(name, category, new_name, new_content, new_category)
        result = db.get_variable(new_category, new_name)
    elif request.method == 'DELETE':
        db.delete_variable(category, name)
        result = []

    result = result
    parameters = {
        'name': name,
        'categories': category,
    }
    return sir.helpers.api.build_api_response(result, error=False, **parameters)


def variables_category(request):
    db = getattr(g, 'db')
    result = db.get_categories()
    parameters = {}
    return sir.helpers.api.build_api_response(result, error=False, **parameters)


def variables_filter_by_category(request, category):
    db = getattr(g, 'db')
    result = db.filter_variables_category(category)
    parameters = {
        'categories': category,
    }
    return sir.helpers.api.build_api_response(result, error=False, **parameters)



import uuid
from setuptools import setup, find_packages
from pip.req import parse_requirements

version = '0.16'

install_reqs = parse_requirements('requirements.txt', session=uuid.uuid1())
reqs = [str(ir.req) for ir in install_reqs]

setup(
    name='SIR',
    version=version,
    packages=find_packages(),
    description='SDN Internet Router',
    long_description=open('README.md').read(),
    author='David Barroso',
    author_email='dbarrosop@dravetech.com',
    license='Apache License, Version 2.0',
    url='https://github.com/dbarrosop/sir',
    install_requires=reqs,
    include_package_data=True,
    zip_safe=False,
)



from flask import g

def build_api_response(result, error=False, **kwargs):
    response = {
        'result': result,
        'parameters': kwargs,
        'meta': {
            'request_time': getattr(g, 'request_time')(),
            'length': len(result),
            'error': error,
        },
    }
    return response
